@@ -2,6 +2,7 @@ import math
 
 from .agent import Agent
 from .communicator import Communicator
+from .interactor import GraspHold
 from .part import ForwardBase, Head
 from .sensor import DistanceSensor, RGBSensor
 
@@ -21,16 +22,16 @@ class HeadAgent(Agent):
         self.distance = DistanceSensor(
             fov=360,
             resolution=36,
-            range=100,
+            max_range=100,
             invisible_elements=self._parts,
             normalize=True,
         )
         self.base.add(self.distance)
 
         self.rgb = RGBSensor(
-            fov=90,
+            fov=180,
             resolution=64,
-            range=400,
+            max_range=400,
             invisible_elements=self._parts,
             invisible_when_grasped=True,
         )
@@ -40,4 +41,6 @@ class HeadAgent(Agent):
         self.comm = Communicator()
         self.base.add(self.comm)
 
-        self.base.add_grasper()
+        # Grapser
+        grasp = GraspHold(base)
+        self.base.add(grasp)
