@@ -1,6 +1,6 @@
 from .ball import Ball
 from .chest import Chest
-from .diamond import Diamond
+from .diamond import Coin, Diamond
 from .disabler import CommunicatorDisabler, ControllerDisabler, Disabler, SensorDisabler
 from .element import (
     PhysicalElement,
@@ -9,12 +9,18 @@ from .element import (
     Teleporter,
     ZoneElement,
 )
-from .wall import BrickWallBlock, ColorWall, WallBlock, create_wall_from_blocks
+from .wall import (
+    ColorWall,
+    TiledAlternateColorWall,
+    TiledGradientColorWall,
+    TiledLongColorWall,
+)
 
 __all__ = [
     "Ball",
     "Chest",
     "Diamond",
+    "Coin",
     "PhysicalElement",
     "RewardElement",
     "SceneElement",
@@ -24,8 +30,8 @@ __all__ = [
     "ControllerDisabler",
     "SensorDisabler",
     "CommunicatorDisabler",
-    "BrickWallBlock",
     "ColorWall",
-    "WallBlock",
-    "create_wall_from_blocks",
+    "TiledLongColorWall",
+    "TiledGradientColorWall",
+    "TiledAlternateColorWall",
 ]
