@@ -27,7 +27,7 @@ from ..agent import Agent
 from ..agent.communicator import Communicator, Message
 from ..agent.controller import Command, Controller
 from ..agent.part import AnchoredPart, PhysicalPart
-from ..agent.sensor import RaySensor, RayShader, Sensor, SensorValue
+from ..agent.sensor import RayCompute, RaySensor, Sensor, SensorValue
 from ..entity import EmbodiedEntity, Entity, InteractiveAnchored
 from ..utils.definitions import (
     PYMUNK_STEPS,
@@ -81,6 +81,7 @@ class Playground:
         background: Optional[
             Union[Tuple[int, int, int], List[int], Tuple[int, int, int, int]]
         ] = None,
+        use_shaders=True,
     ):
 
         # Random number generator for replication, rewind, etc.
@@ -118,7 +119,7 @@ class Playground:
         # Mappings
         self._shapes_to_entities: Dict[pymunk.Shape, EmbodiedEntity] = {}
         self._name_to_agents: Dict[str, Agent] = {}
-        self._uids_to_elements: Dict[int, EmbodiedEntity] = {}
+        self._uids_to_entities: Dict[int, Entity] = {}
 
         self._handle_interactions()
         self._views = []
@@ -127,7 +128,8 @@ class Playground:
         self._window = Window(1, 1, visible=False, antialiasing=True)  # type: ignore
         self._window.ctx.blend_func = self._window.ctx.ONE, self._window.ctx.ZERO
 
-        self._sensor_shader = None
+        self._ray_compute = None
+        self._use_shaders = use_shaders
 
     def debug_draw(self, plt_width=10, center=None, size=None):
 
@@ -164,13 +166,15 @@ class Playground:
         return self._window
 
     @property
-    def sensor_shader(self):
+    def ray_compute(self):
 
-        if not self._sensor_shader:
+        if not self._ray_compute:
             assert self._size
-            self._sensor_shader = RayShader(self, self._size, self._center, zoom=1)
+            self._ray_compute = RayCompute(
+                self, self._size, self._center, zoom=1, use_shader=self._use_shaders
+            )
 
-        return self._sensor_shader
+        return self._ray_compute
 
     @property
     def initial_agent_coordinates(self):
@@ -237,7 +241,7 @@ class Playground:
         while True:
             a = self._rng.integers(0, 2**24)
 
-            if a not in self._uids_to_elements and a != background:
+            if a not in self._uids_to_entities and a != background:
                 uid = a
                 break
 
@@ -250,6 +254,10 @@ class Playground:
 
         return uid, name
 
+    def get_entity_from_uid(self, uid):
+
+        return self._uids_to_entities[uid]
+
     @property
     def agents(self):
         return [agent for agent in self._agents if not agent.removed]
@@ -287,7 +295,6 @@ class Playground:
         self,
         commands: Optional[CommandsDict] = None,
         messages: Optional[SentMessagesDict] = None,
-        skip_state_compute: bool = False,
         pymunk_steps: int = PYMUNK_STEPS,
     ):
         """Update the Playground
@@ -308,24 +315,26 @@ class Playground:
 
         """
 
+        obs, mess, rew = None, None, None
+
         self._pre_step()
-        self._apply_commands(commands)
 
-        for _ in range(pymunk_steps):
-            self.space.step(1.0 / pymunk_steps)
+        if not self._done:
 
-        self._post_step()
+            self._apply_commands(commands)
+
+            for _ in range(pymunk_steps):
+                self.space.step(1.0 / pymunk_steps)
+
+            self._post_step()
+            self._done = self._has_terminated()
 
-        if not skip_state_compute:
             obs = self._compute_observations()
-            rew = self._compute_rewards()
-            mess = None
+            rew = {agent: agent.reward for agent in self._agents}
             if messages:
                 mess = self._transmit_messages(messages)
-        else:
-            obs, mess, rew = None, None, None
 
-        self._timestep += 1
+            self._timestep += 1
 
         return obs, mess, rew, self._done
 
@@ -405,8 +414,8 @@ class Playground:
 
     def _compute_observations(self):
 
-        if self._sensor_shader:
-            self._sensor_shader.update_sensors()
+        if self._ray_compute:
+            self._ray_compute.update_sensors()
 
         obs = {}
         for agent in self.agents:
@@ -414,15 +423,10 @@ class Playground:
 
         return obs
 
-    def _compute_rewards(self):
+    def _has_terminated(self):
+        return False
 
-        rew = {}
-        for agent in self.agents:
-            rew[agent] = agent.compute_rewards()
-
-        return rew
-
-    def reset(self, **kwargs):
+    def reset(self):
         """
         Reset the Playground to its initial state.
         """
@@ -439,13 +443,17 @@ class Playground:
                     element.initial_coordinates,
                     element.allow_overlapping,
                 )
+            elif isinstance(element, EmbodiedEntity) and element.moved:
+                assert element.initial_coordinates
+                element.move_to(
+                    element.initial_coordinates,
+                    element.allow_overlapping,
+                )
 
             element.reset()
 
         for agent in self._agents:
-
             agent.reset()
-
             if agent.removed:
                 self.add(agent, from_removed=True)
             else:
@@ -455,13 +463,15 @@ class Playground:
                     move_anchors=True,
                 )
 
+        for view in self._views:
+            view.update(force=True)
+
         self._timestep = 0
         self._done = False
 
-        obs = self._compute_observations(**kwargs)
-        rew = self._compute_rewards(**kwargs)
+        obs = self._compute_observations()
 
-        return obs, rew, self.done
+        return obs, None, None, self._done
 
     # ADD REMOVE ENTITIES
 
@@ -509,7 +519,7 @@ class Playground:
                 )
 
                 if isinstance(device, RaySensor):
-                    self.sensor_shader.add(device)
+                    self.ray_compute.add(device)
 
         elif isinstance(entity, PhysicalElement):
             for interactive in entity.interactives:
@@ -563,7 +573,7 @@ class Playground:
 
         entity.uid, entity.name = self._get_identifier(entity)
 
-        self._uids_to_elements[entity.uid] = entity
+        self._uids_to_entities[entity.uid] = entity
 
         if isinstance(entity, Agent):
             self._agents.append(entity)
@@ -623,6 +633,9 @@ class Playground:
             for interactive in entity.interactives:
                 self.remove(interactive, definitive)
 
+            for grasper in entity.grasped_by:
+                grasper.release(entity)
+
         entity.removed = True
 
     def _remove_from_space(self, entity):
@@ -639,7 +652,7 @@ class Playground:
 
         assert entity.uid
 
-        self._uids_to_elements.pop(entity.uid)
+        self._uids_to_entities.pop(entity.uid)
 
         if isinstance(entity, Agent):
             self._agents.remove(entity)
@@ -740,6 +753,13 @@ class Playground:
             and elem.shape not in entity.pm_shapes
         ]
 
+        if isinstance(entity, PhysicalPart):
+            agent_shapes = []
+            for part in entity.agent.parts:
+                agent_shapes += part.pm_shapes
+
+            overlaps = [elem for elem in overlaps if elem.shape not in agent_shapes]
+
         self.space.reindex_static()
 
         return bool(overlaps)
